# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update Secure Source Manager repository command."""


from googlecloudsdk.api_lib.securesourcemanager import repositories
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.source_manager import flags
from googlecloudsdk.command_lib.source_manager import resource_args
from googlecloudsdk.core import log

DETAILED_HELP = {
    "DESCRIPTION": """
          Update a Secure Source Manager repository.
        """,
    "EXAMPLES": """
            To update the description of a repository called `my-repo` in
            location `us-central1`, run the following command:

            $ {command} my-repo --description="new description" --region=us-central1
        """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.RegionalEndpointsSupported
class Update(base.UpdateCommand):
  """Update a Secure Source Manager repository."""

  NO_CHANGES_MESSAGE = (
      "There are no changes to the repository [{repository}] for update"
  )

  @staticmethod
  def Args(parser):
    resource_args.AddRepositoryResourceArg(parser, "to update")
    flags.AddDescription(parser)
    flags.AddValidateOnly(parser)
    flags.AddServiceAccount(parser)

  def Run(self, args):
    # Get resource args to contruct base url
    repository_ref = args.CONCEPTS.repository.Parse()
    # Update repository
    client = repositories.RepositoriesClient(
        location=repository_ref.locationsId
    )

    # Collect the list of update masks
    update_mask = []
    if args.IsSpecified("description"):
      update_mask.append("description")
    if args.IsSpecified("service_account"):
      update_mask.append("serviceAccount")

    if not update_mask:
      raise exceptions.MinimumArgumentException(
          [
              "--description",
              "--service-account",
          ],
          self.NO_CHANGES_MESSAGE.format(repository=repository_ref.Name()),
      )

    # this is a shortcut LRO, it completes immediately and is marked as done
    # there is no need to wait
    update_operation = client.Update(
        repository_ref,
        update_mask,
        args.validate_only,
        args.description,
        args.service_account,
    )

    log.UpdatedResource(repository_ref.RelativeName())
    return update_operation


Update.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating VM instances running Docker images."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import containers_utils
from googlecloudsdk.api_lib.compute import image_utils
from googlecloudsdk.api_lib.compute import instance_utils
from googlecloudsdk.api_lib.compute import metadata_utils
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.api_lib.compute.instances.create import utils as create_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute import completers
from googlecloudsdk.command_lib.compute import scope as compute_scopes
from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
from googlecloudsdk.command_lib.compute.resource_policies import flags as maintenance_flags
from googlecloudsdk.command_lib.compute.resource_policies import util as maintenance_util
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


def _Args(
    parser,
    deprecate_maintenance_policy=False,
    container_mount_enabled=False,
    support_multi_writer=True,
    support_confidential_compute_type=False,
    support_confidential_compute_type_tdx=False,
    support_snp_svsm=False,
    support_specific_then_x_affinity=False,
    support_any_reservation_then_fail_affinity=False,
    support_disk_labels=False,
    support_ipv6_only=False,
    support_graceful_shutdown=False,
    support_flex_start=True,
    support_skip_guest_os_shutdown=False,
    support_workload_identity_config=False,
    support_vsock_mode=False,
):
  """Add flags shared by all release tracks."""
  parser.display_info.AddFormat(instances_flags.DEFAULT_LIST_FORMAT)
  metadata_utils.AddMetadataArgs(parser)
  instances_flags.AddDiskArgs(
      parser, True, container_mount_enabled=container_mount_enabled
  )
  instances_flags.AddCreateDiskArgs(
      parser,
      container_mount_enabled=container_mount_enabled,
      support_multi_writer=support_multi_writer,
      support_disk_labels=support_disk_labels,
  )
  instances_flags.AddCanIpForwardArgs(parser)
  instances_flags.AddContainerMountDiskFlag(parser)
  instances_flags.AddAddressArgs(
      parser,
      instances=True,
      containers=True,
      support_ipv6_only=support_ipv6_only,
  )
  instances_flags.AddAcceleratorArgs(parser)
  instances_flags.AddMachineTypeArgs(parser)
  instances_flags.AddMaintenancePolicyArgs(
      parser, deprecate=deprecate_maintenance_policy
  )
  instances_flags.AddNoRestartOnFailureArgs(parser)
  if support_vsock_mode:
    instances_flags.AddVsockModeArgs(parser)
  instances_flags.AddPreemptibleVmArgs(parser)
  instances_flags.AddProvisioningModelVmArgs(
      parser,
      support_flex_start=support_flex_start,
  )
  instances_flags.AddInstanceTerminationActionVmArgs(parser)
  instances_flags.AddServiceAccountAndScopeArgs(parser, False)
  instances_flags.AddTagsArgs(parser)
  instances_flags.AddCustomMachineTypeArgs(parser)
  instances_flags.AddNetworkArgs(parser)
  instances_flags.AddPrivateNetworkIpArgs(parser)
  instances_flags.AddNetworkPerformanceConfigsArgs(parser)
  instances_flags.AddShieldedInstanceConfigArgs(
      parser=parser, for_container=True
  )
  instances_flags.AddKonletArgs(parser)
  instances_flags.AddPublicPtrArgs(parser, instance=True)
  instances_flags.AddImageArgs(parser)
  instances_flags.AddConfidentialComputeArgs(
      parser,
      support_confidential_compute_type,
      support_confidential_compute_type_tdx,
      support_snp_svsm,
  )
  instances_flags.AddNestedVirtualizationArgs(parser)
  instances_flags.AddThreadsPerCoreArgs(parser)
  instances_flags.AddIPv6AddressArgs(parser)
  instances_flags.AddIPv6PrefixLengthArgs(parser)
  labels_util.AddCreateLabelsFlags(parser)
  instances_flags.AddStackTypeArgs(parser, support_ipv6_only)
  instances_flags.AddIpv6NetworkTierArgs(parser)
  instances_flags.AddInternalIPv6AddressArgs(parser)
  instances_flags.AddInternalIPv6PrefixLengthArgs(parser)
  instances_flags.AddMaxRunDurationVmArgs(parser)
  instances_flags.AddDiscardLocalSsdVmArgs(parser)
  if support_graceful_shutdown:
    instances_flags.AddGracefulShutdownArgs(parser, is_create=True)

  instances_flags.AddReservationAffinityGroup(
      parser,
      group_text='Specifies the reservation for the instance.',
      affinity_text='The type of reservation for the instance.',
      support_specific_then_x_affinity=support_specific_then_x_affinity,
      support_any_reservation_then_fail_affinity=support_any_reservation_then_fail_affinity,
  )

  maintenance_flags.AddResourcePoliciesArgs(parser, 'added to', 'instance')

  parser.add_argument(
      '--description', help='Specifies a textual description of the instances.'
  )

  instances_flags.INSTANCES_ARG.AddArgument(parser, operation_type='create')

  CreateWithContainer.SOURCE_INSTANCE_TEMPLATE = (
      instances_flags.MakeSourceInstanceTemplateArg()
  )
  CreateWithContainer.SOURCE_INSTANCE_TEMPLATE.AddArgument(parser)
  parser.display_info.AddCacheUpdater(completers.InstancesCompleter)
  if support_skip_guest_os_shutdown:
    instances_flags.AddSkipGuestOsShutdownArgs(parser)
  if support_workload_identity_config:
    instances_flags.AddWorkloadIdentityConfigArgs(parser)
  instances_flags.AddRequestValidForDurationArgs(parser)


@base.Deprecate(
    is_removed=False,
    warning=(
        'The option to deploy a container during VM creation using the'
        ' container startup agent is deprecated. Use alternative services to'
        ' run containers on your VMs. Learn more at'
        ' https://cloud.google.com/compute/docs/containers/migrate-containers.'
    ),
    error=(
        'The option to deploy a container during VM creation using the'
        ' container startup agent is deprecated. Use alternative services to'
        ' run containers on your VMs. Learn more at'
        ' https://cloud.google.com/compute/docs/containers/migrate-containers.'
    ),
)
# TODO(b/305707759):Change @base.DefaultUniverseOnly to
# @base.UniverseCompatible once b/305707759 is fixed.
# See go/gcloud-cli-running-tpc-tests.
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
class CreateWithContainer(base.CreateCommand):
  """Command for creating VM instances running container images."""

  _container_mount_disk_enabled = True
  _support_create_boot_disk = True
  _support_match_container_mount_disks = True
  _support_nvdimm = False
  _support_host_error_timeout_seconds = True
  _support_numa_node_count = False
  _support_visible_core_count = True
  _support_confidential_compute_type = True
  _support_confidential_compute_type_tdx = True
  _support_snp_svsm = False
  _support_local_ssd_recovery_timeout = True
  _support_specific_then_x_affinity = False
  _support_any_reservation_then_fail_affinity = False
  _support_disk_labels = False
  _support_max_run_duration = True
  _support_graceful_shutdown = True
  _support_skip_guest_os_shutdown = True
  _support_workload_identity_config = False
  _support_vsock_mode = False

  @staticmethod
  def Args(parser):
    """Register parser args."""
    _Args(
        parser,
        container_mount_enabled=True,
        support_multi_writer=False,
        support_confidential_compute_type=True,
        support_confidential_compute_type_tdx=True,
        support_snp_svsm=False,
        support_specific_then_x_affinity=False,
        support_any_reservation_then_fail_affinity=False,
        support_disk_labels=False,
        support_ipv6_only=True,
        support_skip_guest_os_shutdown=True,
        support_workload_identity_config=False,
        support_vsock_mode=False,
    )
    instances_flags.AddNetworkTierArgs(parser, instance=True)
    instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.GA)
    instances_flags.AddPrivateIpv6GoogleAccessArg(
        parser, utils.COMPUTE_GA_API_VERSION
    )
    instances_flags.AddVisibleCoreCountArgs(parser)
    instances_flags.AddLocalSsdRecoveryTimeoutArgs(parser)
    instances_flags.AddHostErrorTimeoutSecondsArgs(parser)

  def _ValidateArgs(self, args):
    self._ValidateTrackSpecificArgs(args)
    instances_flags.ValidateAcceleratorArgs(args)
    instances_flags.ValidateNicFlags(args)
    instances_flags.ValidateNetworkTierArgs(args)
    instances_flags.ValidateKonletArgs(args)
    instances_flags.ValidateDiskCommonFlags(args)
    instances_flags.ValidateServiceAccountAndScopeArgs(args)
    instances_flags.ValidatePublicPtrFlags(args)
    instances_flags.ValidateNetworkPerformanceConfigsArgs(args)
    instances_flags.ValidateInstanceScheduling(
        args, self._support_max_run_duration
    )
    instances_flags.ValidateNetworkTierArgs(args)
    instances_flags.ValidateReservationAffinityGroup(args)

    if instance_utils.UseExistingBootDisk(args.disk or []):
      raise exceptions.InvalidArgumentException(
          '--disk', 'Boot disk specified for containerized VM.'
      )

  def _ValidateTrackSpecificArgs(self, args):
    return None

  def GetImageUri(self, args, compute_client, resource_parser, instance_refs):
    if (
        args.IsSpecified('image')
        or args.IsSpecified('image_family')
        or args.IsSpecified('image_project')
    ):
      image_expander = image_utils.ImageExpander(
          compute_client, resource_parser
      )
      image_uri, _ = image_expander.ExpandImageFlag(
          user_project=instance_refs[0].project,
          image=args.image,
          image_family=args.image_family,
          image_project=args.image_project,
      )
      if resource_parser.Parse(image_uri).project != 'cos-cloud':
        log.warning(
            'This container deployment mechanism requires a '
            'Container-Optimized OS image in order to work. Select an '
            'image from a cos-cloud project (cos-stable, cos-beta, '
            'cos-dev image families).'
        )
    else:
      image_uri = containers_utils.ExpandKonletCosImageFlag(compute_client)
    return image_uri

  def _GetNetworkInterfaces(
      self, args, client, holder, project, location, scope, skip_defaults
  ):
    return create_utils.GetNetworkInterfaces(
        args,
        client,
        holder,
        project,
        location,
        scope,
        skip_defaults,
        support_internal_ipv6_reservation=True,
    )

  def GetNetworkInterfaces(
      self,
      args,
      resources,
      client,
      holder,
      project,
      location,
      scope,
      skip_defaults,
  ):
    if args.network_interface:
      return create_utils.CreateNetworkInterfaceMessages(
          resources=resources,
          compute_client=client,
          network_interface_arg=args.network_interface,
          project=project,
          location=location,
          scope=scope,
          support_internal_ipv6_reservation=True,
      )
    return self._GetNetworkInterfaces(
        args, client, holder, project, location, scope, skip_defaults
    )

  def CheckDiskMessageArgs(self, args, skip_defaults):
    """Creates API messages with disks attached to VM instance."""
    flags_to_check = [
        'create_disk',
        'local_ssd',
        'boot_disk_type',
        'boot_disk_device_name',
        'boot_disk_auto_delete',
        'boot_disk_provisioned_iops',
    ]
    if hasattr(args, 'local_nvdimm'):
      flags_to_check.append('local_nvdimm')
    if (
        skip_defaults
        and not args.IsSpecified('disk')
        and not instance_utils.IsAnySpecified(args, *flags_to_check)
    ):
      return False
    return True

  def Run(self, args):
    self._ValidateArgs(args)

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    compute_client = holder.client
    resource_parser = holder.resources

    container_mount_disk = instances_flags.GetValidatedContainerMountDisk(
        holder, args.container_mount_disk, args.disk, args.create_disk
    )
    source_instance_template = instance_utils.GetSourceInstanceTemplate(
        args, resource_parser, self.SOURCE_INSTANCE_TEMPLATE
    )
    skip_defaults = instance_utils.GetSkipDefaults(source_instance_template)
    scheduling = instance_utils.GetScheduling(
        args,
        compute_client,
        skip_defaults,
        support_min_node_cpu=False,
        support_host_error_timeout_seconds=self._support_host_error_timeout_seconds,
        support_max_run_duration=self._support_max_run_duration,
        support_local_ssd_recovery_timeout=self._support_local_ssd_recovery_timeout,
        support_graceful_shutdown=self._support_graceful_shutdown,
        support_skip_guest_os_shutdown=self._support_skip_guest_os_shutdown,
        support_vsock_mode=self._support_vsock_mode,
    )
    service_accounts = instance_utils.GetServiceAccounts(
        args, compute_client, skip_defaults
    )
    user_metadata = instance_utils.GetValidatedMetadata(args, compute_client)
    boot_disk_size_gb = instance_utils.GetBootDiskSizeGb(args)
    instance_refs = instance_utils.GetInstanceRefs(args, compute_client, holder)
    network_interfaces = self.GetNetworkInterfaces(
        args,
        resource_parser,
        compute_client,
        holder,
        instance_refs[0].project,
        instance_refs[0].zone,
        compute_scopes.ScopeEnum.ZONE,
        skip_defaults,
    )
    image_uri = self.GetImageUri(
        args, compute_client, resource_parser, instance_refs
    )
    labels = containers_utils.GetLabelsMessageWithCosVersion(
        args.labels,
        image_uri,
        resource_parser,
        compute_client.messages.Instance,
    )
    can_ip_forward = instance_utils.GetCanIpForward(args, skip_defaults)
    tags = containers_utils.CreateTagsMessage(
        compute_client.messages, args.tags
    )
    confidential_vm_type = instance_utils.GetConfidentialVmType(
        args, self._support_confidential_compute_type
    )
    workload_identity_config = (
        instance_utils.CreateWorkloadIdentityConfigMessage(
            args,
            compute_client.messages,
            self._support_workload_identity_config,
        )
    )

    requests = []
    for instance_ref in instance_refs:
      metadata = containers_utils.CreateKonletMetadataMessage(
          compute_client.messages,
          args,
          instance_ref.Name(),
          user_metadata,
          container_mount_disk_enabled=self._container_mount_disk_enabled,
          container_mount_disk=container_mount_disk,
      )

      disks = []
      if self.CheckDiskMessageArgs(args, skip_defaults):
        disks = create_utils.CreateDiskMessages(
            args=args,
            instance_name=instance_ref.Name(),
            project=instance_ref.project,
            location=instance_ref.zone,
            scope=compute_scopes.ScopeEnum.ZONE,
            compute_client=compute_client,
            resource_parser=resource_parser,
            boot_disk_size_gb=boot_disk_size_gb,
            image_uri=image_uri,
            create_boot_disk=self._support_create_boot_disk,
            support_nvdimm=self._support_nvdimm,
            support_match_container_mount_disks=self._support_match_container_mount_disks,
            support_disk_labels=self._support_disk_labels,
        )

      machine_type_uri = None
      if instance_utils.CheckSpecifiedMachineTypeArgs(args, skip_defaults):
        machine_type_uri = create_utils.CreateMachineTypeUri(
            args=args,
            compute_client=compute_client,
            resource_parser=resource_parser,
            project=instance_ref.project,
            location=instance_ref.zone,
            scope=compute_scopes.ScopeEnum.ZONE,
            confidential_vm_type=confidential_vm_type,
        )

      guest_accelerators = create_utils.GetAccelerators(
          args=args,
          compute_client=compute_client,
          resource_parser=resource_parser,
          project=instance_ref.project,
          location=instance_ref.zone,
          scope=compute_scopes.ScopeEnum.ZONE,
      )

      instance = compute_client.messages.Instance(
          canIpForward=can_ip_forward,
          disks=disks,
          guestAccelerators=guest_accelerators,
          description=args.description,
          labels=labels,
          machineType=machine_type_uri,
          metadata=metadata,
          minCpuPlatform=args.min_cpu_platform,
          name=instance_ref.Name(),
          networkInterfaces=network_interfaces,
          serviceAccounts=service_accounts,
          scheduling=scheduling,
          tags=tags,
      )

      if self._support_workload_identity_config and workload_identity_config:
        instance.workloadIdentityConfig = workload_identity_config

      if args.private_ipv6_google_access_type is not None:
        instance.privateIpv6GoogleAccess = (
            instances_flags.GetPrivateIpv6GoogleAccessTypeFlagMapper(
                compute_client.messages
            ).GetEnumForChoice(args.private_ipv6_google_access_type)
        )

      if args.IsKnownAndSpecified('request_valid_for_duration'):
        instance.params = instance_utils.CreateParams(args, compute_client)

      confidential_instance_config = create_utils.BuildConfidentialInstanceConfigMessage(
          messages=compute_client.messages,
          args=args,
          support_confidential_compute_type=self._support_confidential_compute_type,
          support_confidential_compute_type_tdx=self._support_confidential_compute_type_tdx,
          support_snp_svsm=self._support_snp_svsm,
      )
      if confidential_instance_config:
        instance.confidentialInstanceConfig = confidential_instance_config

      has_visible_core_count = (
          self._support_visible_core_count
          and args.visible_core_count is not None
      )
      if (
          args.enable_nested_virtualization is not None
          or args.threads_per_core is not None
          or (
              self._support_numa_node_count and args.numa_node_count is not None
          )
          or has_visible_core_count
      ):
        visible_core_count = (
            args.visible_core_count if has_visible_core_count else None
        )
        instance.advancedMachineFeatures = (
            instance_utils.CreateAdvancedMachineFeaturesMessage(
                compute_client.messages,
                args.enable_nested_virtualization,
                args.threads_per_core,
                args.numa_node_count if self._support_numa_node_count else None,
                visible_core_count,
            )
        )

      resource_policies = getattr(args, 'resource_policies', None)
      if resource_policies:
        parsed_resource_policies = []
        for policy in resource_policies:
          resource_policy_ref = maintenance_util.ParseResourcePolicyWithZone(
              resource_parser,
              policy,
              project=instance_ref.project,
              zone=instance_ref.zone,
          )
          parsed_resource_policies.append(resource_policy_ref.SelfLink())
        instance.resourcePolicies = parsed_resource_policies

      shielded_instance_config = (
          create_utils.BuildShieldedInstanceConfigMessage(
              messages=compute_client.messages, args=args
          )
      )
      if shielded_instance_config:
        instance.shieldedInstanceConfig = shielded_instance_config

      if args.IsSpecified('network_performance_configs'):
        instance.networkPerformanceConfig = (
            instance_utils.GetNetworkPerformanceConfig(args, compute_client)
        )

      request = compute_client.messages.ComputeInstancesInsertRequest(
          instance=instance,
          sourceInstanceTemplate=source_instance_template,
          project=instance_ref.project,
          zone=instance_ref.zone,
      )

      request.instance.reservationAffinity = (
          instance_utils.GetReservationAffinity(
              args,
              compute_client,
              self._support_specific_then_x_affinity,
              self._support_any_reservation_then_fail_affinity,
          )
      )
      requests.append(
          (compute_client.apitools_client.instances, 'Insert', request)
      )

    return compute_client.MakeRequests(requests)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateWithContainerBeta(CreateWithContainer):
  """Command for creating VM instances running container images."""

  _container_mount_disk_enabled = True
  _support_create_boot_disk = True
  _support_match_container_mount_disks = True
  _support_nvdimm = False
  _support_visible_core_count = True
  _support_confidential_compute_type = True
  _support_confidential_compute_type_tdx = True
  _support_snp_svsm = False
  _support_host_error_timeout_seconds = True
  _support_numa_node_count = False
  _support_local_ssd_recovery_timeout = True
  _support_specific_then_x_affinity = True
  _support_any_reservation_then_fail_affinity = True
  _support_disk_labels = True
  _support_skip_guest_os_shutdown = True
  _support_workload_identity_config = False

  @staticmethod
  def Args(parser):
    """Register parser args."""
    _Args(
        parser,
        container_mount_enabled=True,
        support_confidential_compute_type=True,
        support_confidential_compute_type_tdx=True,
        support_snp_svsm=False,
        support_specific_then_x_affinity=True,
        support_any_reservation_then_fail_affinity=True,
        support_disk_labels=True,
        support_graceful_shutdown=True,
        support_ipv6_only=True,
        support_flex_start=True,
        support_skip_guest_os_shutdown=True,
        support_workload_identity_config=False,
        support_vsock_mode=False,
    )
    instances_flags.AddNetworkTierArgs(parser, instance=True)
    instances_flags.AddLocalSsdArgs(parser)
    instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.BETA)
    instances_flags.AddPrivateIpv6GoogleAccessArg(
        parser, utils.COMPUTE_BETA_API_VERSION
    )
    instances_flags.AddHostErrorTimeoutSecondsArgs(parser)
    instances_flags.AddVisibleCoreCountArgs(parser)
    instances_flags.AddLocalSsdRecoveryTimeoutArgs(parser)

  def _ValidateTrackSpecificArgs(self, args):
    instances_flags.ValidateLocalSsdFlags(args)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateWithContainerAlpha(CreateWithContainerBeta):
  """Alpha version of compute instances create-with-container command."""

  _container_mount_disk_enabled = True
  _support_create_boot_disk = True
  _support_match_container_mount_disks = True
  _support_nvdimm = True
  _support_host_error_timeout_seconds = True
  _support_numa_node_count = True
  _support_visible_core_count = True
  _support_confidential_compute_type = True
  _support_confidential_compute_type_tdx = True
  _support_snp_svsm = True
  _support_local_ssd_recovery_timeout = True
  _support_specific_then_x_affinity = True
  _support_any_reservation_then_fail_affinity = True
  _support_disk_labels = True
  _support_ipv6_only = True
  _support_skip_guest_os_shutdown = True
  _support_workload_identity_config = True
  _support_vsock_mode = True

  @staticmethod
  def Args(parser):
    _Args(
        parser,
        deprecate_maintenance_policy=True,
        container_mount_enabled=True,
        support_confidential_compute_type=True,
        support_confidential_compute_type_tdx=True,
        support_snp_svsm=True,
        support_specific_then_x_affinity=True,
        support_any_reservation_then_fail_affinity=True,
        support_disk_labels=True,
        support_ipv6_only=True,
        support_graceful_shutdown=True,
        support_flex_start=True,
        support_skip_guest_os_shutdown=True,
        support_workload_identity_config=True,
        support_vsock_mode=True,
    )

    instances_flags.AddNetworkTierArgs(parser, instance=True)
    instances_flags.AddLocalSsdArgsWithSize(parser)
    instances_flags.AddLocalNvdimmArgs(parser)
    instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.ALPHA)
    instances_flags.AddPublicDnsArgs(parser, instance=True)
    instances_flags.AddPrivateIpv6GoogleAccessArg(
        parser, utils.COMPUTE_ALPHA_API_VERSION
    )
    instances_flags.AddHostErrorTimeoutSecondsArgs(parser)
    instances_flags.AddLocalSsdRecoveryTimeoutArgs(parser)
    instances_flags.AddNumaNodeCountArgs(parser)
    instances_flags.AddVisibleCoreCountArgs(parser)
    instances_flags.AddIPv6AddressAlphaArgs(parser)
    instances_flags.AddIPv6PrefixLengthAlphaArgs(parser)

  def _ValidateTrackSpecificArgs(self, args):
    instances_flags.ValidateLocalSsdFlags(args)
    instances_flags.ValidatePublicDnsFlags(args)
    instances_flags.ValidatePublicPtrFlags(args)

  def _GetNetworkInterfaces(
      self, args, client, holder, project, location, scope, skip_defaults
  ):
    return create_utils.GetNetworkInterfacesAlpha(
        args, client, holder, project, location, scope, skip_defaults
    )


CreateWithContainer.detailed_help = {
    'brief': """\
    Creates Compute Engine virtual machine instances running
    container images.
    """,
    'DESCRIPTION': """\
        *{command}* creates Compute Engine virtual
        machines that runs a Docker image. For example:

          $ {command} instance-1 --zone us-central1-a \
            --container-image=gcr.io/google-containers/busybox

        creates an instance called instance-1, in the us-central1-a zone,
        running the 'busybox' image.

        For more examples, refer to the *EXAMPLES* section below.
        """,
    'EXAMPLES': """\
        To run the gcr.io/google-containers/busybox image on an instance named
        'instance-1' that executes 'echo "Hello world"' as a run command, run:

          $ {command} instance-1 \
            --container-image=gcr.io/google-containers/busybox \
            --container-command='echo "Hello world"'

        To run the gcr.io/google-containers/busybox image in privileged mode,
        run:

          $ {command} instance-1 \
            --container-image=gcr.io/google-containers/busybox
            --container-privileged
        """,
}



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a tuning job in Vertex AI."""

from googlecloudsdk.api_lib.ai.tuning_jobs import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import validation
from googlecloudsdk.command_lib.ai.tuning_jobs import flags
from googlecloudsdk.command_lib.ai.tuning_jobs import tuning_jobs_util
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log

_TUNING_JOB_CREATION_DISPLAY_MESSAGE = """\
Tuning job [{id}] submitted successfully.

Your job is still active. You may view the status of your job with the command

 $ gcloud{command_version} ai tuning-jobs describe {id} --region={region}

Job State: {state}\
"""


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA)
class CreateGA(base.CreateCommand):
  r"""Create a supervised fine-tuning job.

    This command creates a new supervised fine-tuning (SFT) tuning job in
    Vertex AI. The job tunes a base foundation model using the provided
    training dataset to produce a customized tuned model.

    ## EXAMPLES

    To create a tuning job that fine-tunes ``gemini-1.0-pro-002'' in region
    ``us-central1'', run:

      $ {command} --region=us-central1 \
        --source-model=gemini-1.0-pro-002 \
        --training-dataset-uri=gs://my-bucket/training.jsonl \
        --tuned-model-display-name=my-tuned-model

    To create a tuning job with labels and hyperparameters:

      $ {command} --region=us-central1 \
        --source-model=gemini-1.0-pro-002 \
        --training-dataset-uri=gs://my-bucket/training.jsonl \
        --validation-dataset-uri=gs://my-bucket/validation.jsonl \
        --epoch-count=3 \
        --learning-rate-multiplier=1.0 \
        --labels=env=prod,team=ml
  """

  _api_version = constants.GA_VERSION
  # Open-model-only flags (`--learning-rate`, `--custom-base-model`,
  # `--output-uri`, `--tuning-mode`, `--batch-size`) are restricted to v1beta1
  # in the proto, so they are only exposed in BETA/ALPHA tracks.
  _include_open_model_flags = False

  @classmethod
  def Args(cls, parser):
    flags.AddCreateTuningJobFlags(
        parser, include_open_model_flags=cls._include_open_model_flags
    )

  def Run(self, args):
    region_ref = args.CONCEPTS.region.Parse()
    region = region_ref.AsDict()["locationsId"]
    validation.ValidateRegion(
        region, available_regions=constants.SUPPORTED_TRAINING_REGIONS
    )

    with endpoint_util.AiplatformEndpointOverrides(
        version=self._api_version, region=region
    ):
      api_client = client.TuningJobsClient(version=self._api_version)

      labels = labels_util.ParseCreateArgs(
          args, api_client.TuningJobMessage().LabelsValue
      )

      adapter_size = api_client.ParseAdapterSize(args.adapter_size)

      # Open-model-only flags may not be defined on the GA argparse namespace
      # because they are gated to v1beta1 tracks. Use getattr to read them
      # safely so the GA Run() can share the same body as BETA/ALPHA.
      learning_rate = getattr(args, "learning_rate", None)
      batch_size = getattr(args, "batch_size", None)
      custom_base_model = getattr(args, "custom_base_model", None)
      output_uri = getattr(args, "output_uri", None)
      tuning_mode = api_client.ParseTuningMode(
          getattr(args, "tuning_mode", None)
      )

      response = api_client.Create(
          parent=region_ref.RelativeName(),
          source_model=args.source_model,
          training_dataset_uri=args.training_dataset_uri,
          validation_dataset_uri=args.validation_dataset_uri,
          tuned_model_display_name=args.tuned_model_display_name,
          description=args.description,
          epoch_count=args.epoch_count,
          learning_rate_multiplier=args.learning_rate_multiplier,
          learning_rate=learning_rate,
          batch_size=batch_size,
          adapter_size=adapter_size,
          tuning_mode=tuning_mode,
          export_last_checkpoint_only=args.export_last_checkpoint_only,
          custom_base_model=custom_base_model,
          output_uri=output_uri,
          labels=labels,
          encryption_key_name=validation.GetAndValidateKmsKey(args),
          service_account=args.service_account,
      )

      log.status.Print(
          _TUNING_JOB_CREATION_DISPLAY_MESSAGE.format(
              id=tuning_jobs_util.ParseJobName(response.name),
              command_version=tuning_jobs_util.OutputCommandVersion(
                  self.ReleaseTrack()
              ),
              region=region,
              state=response.state,
          )
      )
      return response


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class CreatePreGA(CreateGA):
  """Create a supervised fine-tuning job."""

  _api_version = constants.BETA_VERSION
  _include_open_model_flags = True



"""Generated message classes for dataprocgdc version v1alpha1.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'dataprocgdc'


class ApplicationEnvironment(_messages.Message):
  r"""Represents the ApplicationEnvironment resource.

  Messages:
    AnnotationsValue: Optional. The annotations to associate with this
      application environment. Annotations may be used to store client
      information, but are not used by the server.
    LabelsValue: Optional. The labels to associate with this application
      environment. Labels may be used for filtering and billing tracking.

  Fields:
    annotations: Optional. The annotations to associate with this application
      environment. Annotations may be used to store client information, but
      are not used by the server.
    createTime: Output only. The timestamp when the resource was created.
    displayName: Optional. User-provided human-readable name to be used in
      user interfaces.
    labels: Optional. The labels to associate with this application
      environment. Labels may be used for filtering and billing tracking.
    name: Identifier. Fields 1-6 should exist for all declarative friendly
      resources per aip.dev/148 The name of the application environment.
      Format: projects/{project}/locations/{location}/serviceInstances/{servic
      e_instance}/applicationEnvironments/{application_environment_id}
    namespace: Optional. The name of the namespace in which to create this
      ApplicationEnvironment. This namespace must already exist in the cluster
    sparkApplicationEnvironmentConfig: Optional. The engine-specific
      configurations for this ApplicationEnvironment.
    uid: Output only. System generated unique identifier for this application
      environment, formatted as UUID4.
    updateTime: Output only. The timestamp when the resource was most recently
      updated.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AnnotationsValue(_messages.Message):
    r"""Optional. The annotations to associate with this application
    environment. Annotations may be used to store client information, but are
    not used by the server.

    Messages:
      AdditionalProperty: An additional property for a AnnotationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type AnnotationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AnnotationsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. The labels to associate with this application environment.
    Labels may be used for filtering and billing tracking.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  annotations = _messages.MessageField('AnnotationsValue', 1)
  createTime = _messages.StringField(2)
  displayName = _messages.StringField(3)
  labels = _messages.MessageField('LabelsValue', 4)
  name = _messages.StringField(5)
  namespace = _messages.StringField(6)
  sparkApplicationEnvironmentConfig = _messages.MessageField('SparkApplicationEnvironmentConfig', 7)
  uid = _messages.StringField(8)
  updateTime = _messages.StringField(9)


class AuxiliaryServicesConfig(_messages.Message):
  r"""Auxiliary Service Configs.

  Fields:
    sparkHistoryServer: Optional. Spark History Servor message.
  """

  sparkHistoryServer = _messages.MessageField('SparkHistoryServer', 1)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class DataprocgdcProjectsLocationsGetRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class DataprocgdcProjectsLocationsListRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class DataprocgdcProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class DataprocgdcProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class DataprocgdcProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class DataprocgdcProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsCreateRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsCre
  ateRequest object.

  Fields:
    applicationEnvironment: A ApplicationEnvironment resource to be passed as
      the request body.
    applicationEnvironmentId: Optional. The id of the application environment
    parent: Required. The parent location resource where this application
      environment will be created
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  applicationEnvironment = _messages.MessageField('ApplicationEnvironment', 1)
  applicationEnvironmentId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsDeleteRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsDel
  eteRequest object.

  Fields:
    allowMissing: Optional. If set to true, and the application is not found,
      the request will succeed but no action will be taken on the server
    etag: Optional. The etag of the application. If this is provided, it must
      match the server etag.
    name: Required. The name of the application to delete.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  allowMissing = _messages.BooleanField(1)
  etag = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsGetRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsGet
  Request object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsListRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsLis
  tRequest object.

  Fields:
    filter: Optional. Filtering results
    orderBy: Optional. Hint for how to order the results
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Parent value for ListSparkApplicationsRequest
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsPatchRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesApplicationEnvironmentsPat
  chRequest object.

  Fields:
    allowMissing: Optional. Whether to upsert this application if it does not
      exist already In this situation, `update_mask` is ignored.
    applicationEnvironment: A ApplicationEnvironment resource to be passed as
      the request body.
    name: Identifier. Fields 1-6 should exist for all declarative friendly
      resources per aip.dev/148 The name of the application environment.
      Format: projects/{project}/locations/{location}/serviceInstances/{servic
      e_instance}/applicationEnvironments/{application_environment_id}
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Required. The list of fields to update.
  """

  allowMissing = _messages.BooleanField(1)
  applicationEnvironment = _messages.MessageField('ApplicationEnvironment', 2)
  name = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)
  updateMask = _messages.StringField(5)


class DataprocgdcProjectsLocationsServiceInstancesCreateRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesCreateRequest object.

  Fields:
    parent: Required. Value for parent.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    serviceInstance: A ServiceInstance resource to be passed as the request
      body.
    serviceInstanceId: Required. Id of the requesting object If auto-
      generating Id server-side, remove this field and service_instance_id
      from the method_signature of Create RPC
  """

  parent = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  serviceInstance = _messages.MessageField('ServiceInstance', 3)
  serviceInstanceId = _messages.StringField(4)


class DataprocgdcProjectsLocationsServiceInstancesDeleteRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesDeleteRequest object.

  Fields:
    allowMissing: Optional. If set to true, and the service instance is not
      found, the request will succeed but no action will be taken on the
      server
    etag: Optional. The etag of the service instance. If this is provided, it
      must match the server etag.
    force: Optional. If set to true, any jobs and job environments associated
      with this service instance will also be deleted. If false (default) the
      service instance can only be deleted if there are no job environments or
      jobs associated with the service instance.
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  allowMissing = _messages.BooleanField(1)
  etag = _messages.StringField(2)
  force = _messages.BooleanField(3)
  name = _messages.StringField(4, required=True)
  requestId = _messages.StringField(5)


class DataprocgdcProjectsLocationsServiceInstancesGetRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesGetRequest object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class DataprocgdcProjectsLocationsServiceInstancesListRequest(_messages.Message):
  r"""A DataprocgdcProjectsLocationsServiceInstancesListRequest object.

  Fields:
    filter: Optional. Filtering results
    orderBy: Optional. Hint for how to order the results
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Parent value for ListServiceInstancesRequest
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsCreateRequest(_messages.Message):
  r"""A
  DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsCreateRequest
  object.

  Fields:
    parent: Required. The parent location resource where this application will
      be created
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    sparkApplication: A SparkApplication resource to be passed as the request
      body.
    sparkApplicationId: Required. The id of the application
  """

  parent = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  sparkApplication = _messages.MessageField('SparkApplication', 3)
  sparkApplicationId = _messages.StringField(4)


class DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsDeleteRequest(_messages.Message):
  r"""A
  DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsDeleteRequest
  object.

  Fields:
    allowMissing: Optional. If set to true, and the application is not found,
      the request will succeed but no action will be taken on the server
    etag: Optional. The etag of the application. If this is provided, it must
      match the server etag.
    name: Required. The name of the application to delete.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  allowMissing = _messages.BooleanField(1)
  etag = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsGetRequest(_messages.Message):
  r"""A
  DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsGetRequest
  object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsListRequest(_messages.Message):
  r"""A
  DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsListRequest
  object.

  Fields:
    filter: Optional. Filtering results
    orderBy: Optional. Hint for how to order the results
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Parent value for ListSparkApplicationsRequest
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsPatchRequest(_messages.Message):
  r"""A
  DataprocgdcProjectsLocationsServiceInstancesSparkApplicationsPatchRequest
  object.

  Fields:
    allowMissing: Optional. Whether to upsert this application if it does not
      exist already In this situation, `update_mask` is ignored.
    name: Identifier. Fields 1-6 should exist for all declarative friendly
      resources per aip.dev/148 The name of the application. Format: projects/
      {project}/locations/{location}/serviceInstances/{service_instance}/spark
      Applications/{application}
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    sparkApplication: A SparkApplication resource to be passed as the request
      body.
    updateMask: Required. The list of fields to update.
  """

  allowMissing = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  sparkApplication = _messages.MessageField('SparkApplication', 4)
  updateMask = _messages.StringField(5)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class GdceCluster(_messages.Message):
  r"""Gdce cluster information.

  Fields:
    gdceCluster: Required. Gdce cluster resource id.
  """

  gdceCluster = _messages.StringField(1)


class ListApplicationEnvironmentsResponse(_messages.Message):
  r"""Message for response to listing ApplicationEnvironments

  Fields:
    applicationEnvironments: The list of ApplicationEnvironment
    nextPageToken: Output only. A token identifying a page of results the
      server should return.
    unreachable: Output only. Locations that could not be reached.
  """

  applicationEnvironments = _messages.MessageField('ApplicationEnvironment', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListServiceInstancesResponse(_messages.Message):
  r"""Message for response to listing ServiceInstances

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    serviceInstances: The list of ServiceInstance
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  serviceInstances = _messages.MessageField('ServiceInstance', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListSparkApplicationsResponse(_messages.Message):
  r"""Message for response to listing SparkApplications

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    sparkApplications: The list of SparkApplication
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  sparkApplications = _messages.MessageField('SparkApplication', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class MaintenancePolicy(_messages.Message):
  r"""Maintenance policy for a service instance.

  Fields:
    maintenanceWindow: Optional. The maintenance window for the service
      instance.
  """

  maintenanceWindow = _messages.MessageField('MaintenanceWindow', 1)


class MaintenanceWindow(_messages.Message):
  r"""Maintenance window for a service instance.

  Enums:
    DayOfWeekValueValuesEnum: Optional. The day of the week when maintenance
      is scheduled.

  Fields:
    dayOfWeek: Optional. The day of the week when maintenance is scheduled.
    duration: Required. Duration of the time window, set by service producer.
    startTime: Optional. Time within the window to start the operations.
  """

  class DayOfWeekValueValuesEnum(_messages.Enum):
    r"""Optional. The day of the week when maintenance is scheduled.

    Values:
      DAY_OF_WEEK_UNSPECIFIED: The day of the week is unspecified.
      MONDAY: Monday
      TUESDAY: Tuesday
      WEDNESDAY: Wednesday
      THURSDAY: Thursday
      FRIDAY: Friday
      SATURDAY: Saturday
      SUNDAY: Sunday
    """
    DAY_OF_WEEK_UNSPECIFIED = 0
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7

  dayOfWeek = _messages.EnumField('DayOfWeekValueValuesEnum', 1)
  duration = _messages.StringField(2)
  startTime = _messages.MessageField('TimeOfDay', 3)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class PySparkApplicationConfig(_messages.Message):
  r"""Represents the PySparkApplicationConfig.

  Fields:
    archiveUris: Optional. HCFS URIs of archives to be extracted into the
      working directory of each executor. Supported file types: .jar, .tar,
      .tar.gz, .tgz, and .zip.
    args: Optional. The arguments to pass to the driver. Do not include
      arguments, such as `--conf`, that can be set as job properties, since a
      collision may occur that causes an incorrect job submission.
    fileUris: Optional. HCFS URIs of files to be placed in the working
      directory of each executor. Useful for naively parallel tasks.
    jarFileUris: Optional. HCFS URIs of jar files to add to the CLASSPATHs of
      the Python driver and tasks.
    mainPythonFileUri: Required. The HCFS URI of the main Python file to use
      as the driver. Must be a .py file.
    pythonFileUris: Optional. HCFS file URIs of Python files to pass to the
      PySpark framework. Supported file types: .py, .egg, and .zip.
  """

  archiveUris = _messages.StringField(1, repeated=True)
  args = _messages.StringField(2, repeated=True)
  fileUris = _messages.StringField(3, repeated=True)
  jarFileUris = _messages.StringField(4, repeated=True)
  mainPythonFileUri = _messages.StringField(5)
  pythonFileUris = _messages.StringField(6, repeated=True)


class QueryList(_messages.Message):
  r"""Represents a list of queries.

  Fields:
    queries: Required. The queries to run.
  """

  queries = _messages.StringField(1, repeated=True)


class ServiceInstance(_messages.Message):
  r"""Message describing ServiceInstance object TODO(user) add appropriate
  visibility tags to the fields of this proto.

  Enums:
    RequestedStateValueValuesEnum: Output only. The intended state to which
      the service instance is reconciling.
    StateValueValuesEnum: Output only. The current state.

  Messages:
    AnnotationsValue: Optional. The annotations to associate with this service
      instance. Annotations may be used to store client information, but are
      not used by the server.
    LabelsValue: Optional. The labels to associate with this service instance.
      Labels may be used for filtering and billing tracking.

  Fields:
    annotations: Optional. The annotations to associate with this service
      instance. Annotations may be used to store client information, but are
      not used by the server.
    auxiliaryServicesConfig: Optional. Configuration of auxiliary services
      used by this instance.
    createTime: Output only. The timestamp when the resource was created.
    displayName: Optional. User-provided human-readable name to be used in
      user interfaces.
    effectiveServiceAccount: Output only. Effective service account associated
      with ServiceInstance. This will be the service_account if specified.
      Otherwise, it will be an automatically created per-resource P4SA that
      also automatically has Fleet Workload Identity bindings applied.
    gdceCluster: Optional. A GDCE cluster.
    labels: Optional. The labels to associate with this service instance.
      Labels may be used for filtering and billing tracking.
    maintenancePolicy: Optional. Maintenance policy for this service instance.
    name: Identifier. The name of the service instance.
    reconciling: Output only. Whether the service instance is currently
      reconciling. True if the current state of the resource does not match
      the intended state, and the system is working to reconcile them, whether
      or not the change was user initiated. Required by
      aip.dev/128#reconciliation
    requestedState: Output only. The intended state to which the service
      instance is reconciling.
    serviceAccount: Optional. Requested service account to associate with
      ServiceInstance.
    sparkServiceInstanceConfig: Optional. Spark-specific service instance
      configuration.
    state: Output only. The current state.
    stateMessage: Output only. A message explaining the current state.
    uid: Output only. System generated unique identifier for this service
      instance, formatted as UUID4.
    updateTime: Output only. The timestamp when the resource was most recently
      updated.
  """

  class RequestedStateValueValuesEnum(_messages.Enum):
    r"""Output only. The intended state to which the service instance is
    reconciling.

    Values:
      STATE_UNSPECIFIED: The service instance state is unknown.
      CREATING: The service instance is being created and is not yet ready to
        accept requests.
      ACTIVE: The service instance is running.
      DISCONNECTED: The service instance is running but disconnected from the
        Google network
      DELETING: The service instance is being deleted.
      STOPPING: The service instance is being stopped. Not in scope for
        private GA
      STOPPED: The service instance is stopped. Not in scope for private GA
      STARTING: The service instance is being started from being STOPPED. Not
        in scope for private GA
      UPDATING: The service instance is being updated
      FAILED: The service instance has encountered an unrecoverable error.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DISCONNECTED = 3
    DELETING = 4
    STOPPING = 5
    STOPPED = 6
    STARTING = 7
    UPDATING = 8
    FAILED = 9

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current state.

    Values:
      STATE_UNSPECIFIED: The service instance state is unknown.
      CREATING: The service instance is being created and is not yet ready to
        accept requests.
      ACTIVE: The service instance is running.
      DISCONNECTED: The service instance is running but disconnected from the
        Google network
      DELETING: The service instance is being deleted.
      STOPPING: The service instance is being stopped. Not in scope for
        private GA
      STOPPED: The service instance is stopped. Not in scope for private GA
      STARTING: The service instance is being started from being STOPPED. Not
        in scope for private GA
      UPDATING: The service instance is being updated
      FAILED: The service instance has encountered an unrecoverable error.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DISCONNECTED = 3
    DELETING = 4
    STOPPING = 5
    STOPPED = 6
    STARTING = 7
    UPDATING = 8
    FAILED = 9

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AnnotationsValue(_messages.Message):
    r"""Optional. The annotations to associate with this service instance.
    Annotations may be used to store client information, but are not used by
    the server.

    Messages:
      AdditionalProperty: An additional property for a AnnotationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type AnnotationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AnnotationsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. The labels to associate with this service instance. Labels
    may be used for filtering and billing tracking.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  annotations = _messages.MessageField('AnnotationsValue', 1)
  auxiliaryServicesConfig = _messages.MessageField('AuxiliaryServicesConfig', 2)
  createTime = _messages.StringField(3)
  displayName = _messages.StringField(4)
  effectiveServiceAccount = _messages.StringField(5)
  gdceCluster = _messages.MessageField('GdceCluster', 6)
  labels = _messages.MessageField('LabelsValue', 7)
  maintenancePolicy = _messages.MessageField('MaintenancePolicy', 8)
  name = _messages.StringField(9)
  reconciling = _messages.BooleanField(10)
  requestedState = _messages.EnumField('RequestedStateValueValuesEnum', 11)
  serviceAccount = _messages.StringField(12)
  sparkServiceInstanceConfig = _messages.MessageField('SparkServiceInstanceConfig', 13)
  state = _messages.EnumField('StateValueValuesEnum', 14)
  stateMessage = _messages.StringField(15)
  uid = _messages.StringField(16)
  updateTime = _messages.StringField(17)


class SparkApplication(_messages.Message):
  r"""Represents the SparkApplication resource.

  Enums:
    RequestedStateValueValuesEnum: Optional. The intended state to which the
      application is reconciling.
    StateValueValuesEnum: Output only. The current state.

  Messages:
    AnnotationsValue: Optional. The annotations to associate with this
      application. Annotations may be used to store client information, but
      are not used by the server.
    LabelsValue: Optional. The labels to associate with this application.
      Labels may be used for filtering and billing tracking.
    PropertiesValue: Optional. application-specific properties.

  Fields:
    annotations: Optional. The annotations to associate with this application.
      Annotations may be used to store client information, but are not used by
      the server.
    applicationEnvironment: Optional. An ApplicationEnvironment from which to
      inherit configuration properties.
    createTime: Output only. The timestamp when the resource was created.
    dependencyImages: Optional. List of container image uris for additional
      file dependencies. Dependent files are sequentially copied from each
      image. If a file with the same name exists in 2 images then the file
      from later image is used.
    displayName: Optional. User-provided human-readable name to be used in
      user interfaces.
    labels: Optional. The labels to associate with this application. Labels
      may be used for filtering and billing tracking.
    monitoringEndpoint: Output only. URL for a monitoring UI for this
      application (for eventual Spark PHS/UI support) Out of scope for private
      GA
    name: Identifier. Fields 1-6 should exist for all declarative friendly
      resources per aip.dev/148 The name of the application. Format: projects/
      {project}/locations/{location}/serviceInstances/{service_instance}/spark
      Applications/{application}
    namespace: Optional. The Kubernetes namespace in which to create the
      application. This namespace must already exist on the cluster.
    outputUri: Output only. An HCFS URI pointing to the location of stdout and
      stdout of the application Mainly useful for Pantheon and gcloud Not in
      scope for private GA
    properties: Optional. application-specific properties.
    pysparkApplicationConfig: PySpark application config.
    reconciling: Output only. Whether the application is currently
      reconciling. True if the current state of the resource does not match
      the intended state, and the system is working to reconcile them, whether
      or not the change was user initiated. Required by
      aip.dev/128#reconciliation
    requestedState: Optional. The intended state to which the application is
      reconciling.
    sparkApplicationConfig: Spark application config.
    sparkRApplicationConfig: SparkR application config.
    sparkSqlApplicationConfig: SparkSql application config.
    state: Output only. The current state.
    stateMessage: Output only. A message explaining the current state.
    uid: Output only. System generated unique identifier for this application,
      formatted as UUID4.
    updateTime: Output only. The timestamp when the resource was most recently
      updated.
    version: Optional. The Dataproc version of this application.
  """

  class RequestedStateValueValuesEnum(_messages.Enum):
    r"""Optional. The intended state to which the application is reconciling.

    Values:
      STATE_UNSPECIFIED: The application state is unknown.
      PENDING: The application is setting up and has not yet begun to execute
      RUNNING: The application is running.
      CANCELLING: The application is being cancelled.
      CANCELLED: The application was successfully cancelled
      SUCCEEDED: The application completed successfully.
      FAILED: The application exited with an error.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    RUNNING = 2
    CANCELLING = 3
    CANCELLED = 4
    SUCCEEDED = 5
    FAILED = 6

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current state.

    Values:
      STATE_UNSPECIFIED: The application state is unknown.
      PENDING: The application is setting up and has not yet begun to execute
      RUNNING: The application is running.
      CANCELLING: The application is being cancelled.
      CANCELLED: The application was successfully cancelled
      SUCCEEDED: The application completed successfully.
      FAILED: The application exited with an error.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    RUNNING = 2
    CANCELLING = 3
    CANCELLED = 4
    SUCCEEDED = 5
    FAILED = 6

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AnnotationsValue(_messages.Message):
    r"""Optional. The annotations to associate with this application.
    Annotations may be used to store client information, but are not used by
    the server.

    Messages:
      AdditionalProperty: An additional property for a AnnotationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type AnnotationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AnnotationsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. The labels to associate with this application. Labels may be
    used for filtering and billing tracking.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class PropertiesValue(_messages.Message):
    r"""Optional. application-specific properties.

    Messages:
      AdditionalProperty: An additional property for a PropertiesValue object.

    Fields:
      additionalProperties: Additional properties of type PropertiesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a PropertiesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  annotations = _messages.MessageField('AnnotationsValue', 1)
  applicationEnvironment = _messages.StringField(2)
  createTime = _messages.StringField(3)
  dependencyImages = _messages.StringField(4, repeated=True)
  displayName = _messages.StringField(5)
  labels = _messages.MessageField('LabelsValue', 6)
  monitoringEndpoint = _messages.StringField(7)
  name = _messages.StringField(8)
  namespace = _messages.StringField(9)
  outputUri = _messages.StringField(10)
  properties = _messages.MessageField('PropertiesValue', 11)
  pysparkApplicationConfig = _messages.MessageField('PySparkApplicationConfig', 12)
  reconciling = _messages.BooleanField(13)
  requestedState = _messages.EnumField('RequestedStateValueValuesEnum', 14)
  sparkApplicationConfig = _messages.MessageField('SparkApplicationConfig', 15)
  sparkRApplicationConfig = _messages.MessageField('SparkRApplicationConfig', 16)
  sparkSqlApplicationConfig = _messages.MessageField('SparkSqlApplicationConfig', 17)
  state = _messages.EnumField('StateValueValuesEnum', 18)
  stateMessage = _messages.StringField(19)
  uid = _messages.StringField(20)
  updateTime = _messages.StringField(21)
  version = _messages.StringField(22)


class SparkApplicationConfig(_messages.Message):
  r"""Represents the SparkApplicationConfig.

  Fields:
    archiveUris: Optional. HCFS URIs of archives to be extracted into the
      working directory of each executor. Supported file types: `.jar`,
      `.tar`, `.tar.gz`, `.tgz`, and `.zip`.
    args: Optional. The arguments to pass to the driver. Do not include
      arguments that can be set as application properties, such as `--conf`,
      since a collision can occur that causes an incorrect application
      submission.
    fileUris: Optional. HCFS URIs of files to be placed in the working
      directory of each executor.
    jarFileUris: Optional. HCFS URIs of jar files to add to the classpath of
      the Spark driver and tasks.
    mainClass: The name of the driver main class. The jar file that contains
      the class must be in the classpath or specified in `jar_file_uris`.
    mainJarFileUri: The HCFS URI of the jar file that contains the main class.
  """

  archiveUris = _messages.StringField(1, repeated=True)
  args = _messages.StringField(2, repeated=True)
  fileUris = _messages.StringField(3, repeated=True)
  jarFileUris = _messages.StringField(4, repeated=True)
  mainClass = _messages.StringField(5)
  mainJarFileUri = _messages.StringField(6)


class SparkApplicationEnvironmentConfig(_messages.Message):
  r"""Represents the SparkApplicationEnvironmentConfig.

  Messages:
    DefaultPropertiesValue: Optional. A map of default Spark properties to
      apply to workloads in this application environment. These defaults may
      be overridden by per-application properties.

  Fields:
    defaultProperties: Optional. A map of default Spark properties to apply to
      workloads in this application environment. These defaults may be
      overridden by per-application properties.
    defaultVersion: Optional. The default Dataproc version to use for
      applications submitted to this application environment
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DefaultPropertiesValue(_messages.Message):
    r"""Optional. A map of default Spark properties to apply to workloads in
    this application environment. These defaults may be overridden by per-
    application properties.

    Messages:
      AdditionalProperty: An additional property for a DefaultPropertiesValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        DefaultPropertiesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DefaultPropertiesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  defaultProperties = _messages.MessageField('DefaultPropertiesValue', 1)
  defaultVersion = _messages.StringField(2)


class SparkApplicationOperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class SparkHistoryServer(_messages.Message):
  r"""Spark History Server.

  Fields:
    sparkHistoryServerConfig: Optional. Spark History Server configurations
      for a given version.
  """

  sparkHistoryServerConfig = _messages.MessageField('SparkHistoryServerConfig', 1)


class SparkHistoryServerConfig(_messages.Message):
  r"""Spark History Server Config.

  Messages:
    ConfigurationsValue: Optional. Mapping of configurations.

  Fields:
    configurations: Optional. Mapping of configurations.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ConfigurationsValue(_messages.Message):
    r"""Optional. Mapping of configurations.

    Messages:
      AdditionalProperty: An additional property for a ConfigurationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type ConfigurationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ConfigurationsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  configurations = _messages.MessageField('ConfigurationsValue', 1)


class SparkRApplicationConfig(_messages.Message):
  r"""Represents the SparkRApplicationConfig.

  Fields:
    archiveUris: Optional. HCFS URIs of archives to be extracted into the
      working directory of each executor. Supported file types: .jar, .tar,
      .tar.gz, .tgz, and .zip.
    args: Optional. The arguments to pass to the driver. Do not include
      arguments, such as `--conf`, that can be set as job properties, since a
      collision may occur that causes an incorrect job submission.
    fileUris: Optional. HCFS URIs of files to be placed in the working
      directory of each executor. Useful for naively parallel tasks.
    mainRFileUri: Required. The HCFS URI of the main R file to use as the
      driver. Must be a .R file.
  """

  archiveUris = _messages.StringField(1, repeated=True)
  args = _messages.StringField(2, repeated=True)
  fileUris = _messages.StringField(3, repeated=True)
  mainRFileUri = _messages.StringField(4)


class SparkServiceInstanceConfig(_messages.Message):
  r"""Spark-specific service instance configuration."""


class SparkSqlApplicationConfig(_messages.Message):
  r"""Represents the SparkRApplicationConfig.

  Messages:
    ScriptVariablesValue: Optional. Mapping of query variable names to values
      (equivalent to the Spark SQL command: SET `name="value";`).

  Fields:
    jarFileUris: Optional. HCFS URIs of jar files to be added to the Spark
      CLASSPATH.
    queryFileUri: The HCFS URI of the script that contains SQL queries.
    queryList: A list of queries.
    scriptVariables: Optional. Mapping of query variable names to values
      (equivalent to the Spark SQL command: SET `name="value";`).
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ScriptVariablesValue(_messages.Message):
    r"""Optional. Mapping of query variable names to values (equivalent to the
    Spark SQL command: SET `name="value";`).

    Messages:
      AdditionalProperty: An additional property for a ScriptVariablesValue
        object.

    Fields:
      additionalProperties: Additional properties of type ScriptVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ScriptVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  jarFileUris = _messages.StringField(1, repeated=True)
  queryFileUri = _messages.StringField(2)
  queryList = _messages.MessageField('QueryList', 3)
  scriptVariables = _messages.MessageField('ScriptVariablesValue', 4)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TimeOfDay(_messages.Message):
  r"""Represents a time of day. The date and time zone are either not
  significant or are specified elsewhere. An API may choose to allow leap
  seconds. Related types are google.type.Date and `google.protobuf.Timestamp`.

  Fields:
    hours: Hours of a day in 24 hour format. Must be greater than or equal to
      0 and typically must be less than or equal to 23. An API may choose to
      allow the value "24:00:00" for scenarios like business closing time.
    minutes: Minutes of an hour. Must be greater than or equal to 0 and less
      than or equal to 59.
    nanos: Fractions of seconds, in nanoseconds. Must be greater than or equal
      to 0 and less than or equal to 999,999,999.
    seconds: Seconds of a minute. Must be greater than or equal to 0 and
      typically must be less than or equal to 59. An API may allow the value
      60 if it allows leap-seconds.
  """

  hours = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  minutes = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  nanos = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  seconds = _messages.IntegerField(4, variant=_messages.Variant.INT32)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Flag type definitions for clusters command group."""

from typing import Any
from googlecloudsdk.api_lib.hypercomputecluster import utils as api_utils
from googlecloudsdk.calliope import arg_parsers


UPDATE_MASK_OBJECT = arg_parsers.ArgObject(
    value_type=str, enable_shorthand=True
)

NETWORK_OBJECT = arg_parsers.ArgObject(
    spec={
        "name": str,
        "description": str,
    },
    required_keys=["name"],
    enable_shorthand=True,
    disable_key_description=True,
)

LABEL = arg_parsers.ArgObject(
    key_type=str,
    value_type=str,
    enable_shorthand=True,
)

STORAGE_CONFIG = arg_parsers.ArgObject(
    spec={
        "id": str,
        "localMount": str,
    },
    required_keys=["id", "localMount"],
    repeated=True,
)

EXISTING_FILESTORES_TYPE = arg_parsers.ArgObject(
    spec={
        "id": str,
        "name": str,
    },
    required_keys=["id", "name"],
    enable_shorthand=True,
    repeated=True,
)

EXISTING_BUCKETS_TYPE = arg_parsers.ArgObject(
    spec={
        "id": str,
        "name": str,
    },
    required_keys=["id", "name"],
    enable_shorthand=True,
    repeated=True,
)

EXISTING_LUSTRES_TYPE = arg_parsers.ArgObject(
    spec={
        "id": str,
        "name": str,
    },
    required_keys=["id", "name"],
    enable_shorthand=True,
    repeated=True,
)

LUSTRES_OBJECT = arg_parsers.ArgObject(
    spec={
        "id": str,
        "name": str,
        "filesystem": str,
        "capacityGb": int,
        "description": str,
        "perUnitStorageThroughput": int,
    },
    required_keys=["id", "name", "capacityGb", "filesystem"],
    repeated=True,
    enable_shorthand=True,
    disable_key_description=True,
)


SERVICE_ACCOUNT_TYPE = arg_parsers.ArgObject(
    spec={
        "email": str,
        "scopes": arg_parsers.ArgList(),
    }
)

SLURM_CONFIG_TYPE = arg_parsers.ArgObject(
    spec={
        "requeueExitCodes": arg_parsers.ArgList(element_type=int),
        "requeueHoldExitCodes": arg_parsers.ArgList(element_type=int),
        "prologFlags": arg_parsers.ArgList(element_type=str),
        "prologEpilogTimeout": str,
        "accountingStorageEnforceFlags": arg_parsers.ArgList(element_type=str),
        "priorityType": str,
        "priorityWeightAge": int,
        "priorityWeightAssoc": int,
        "priorityWeightFairshare": int,
        "priorityWeightJobSize": int,
        "priorityWeightPartition": int,
        "priorityWeightQos": int,
        "priorityWeightTres": str,
        "preemptMode": arg_parsers.ArgList(element_type=str),
        "preemptType": str,
        "preemptExemptTime": str,
        "healthCheckInterval": int,
        "healthCheckNodeState": str,
        "healthCheckProgram": str,
    }
)
FILESTORE_CONFIG_SPEC = {
    "id": str,
    "description": str,
    "fileShares": arg_parsers.ArgObject(
        spec={"capacityGb": int, "fileShare": str},
        repeated=True,
    ),
    "filestore": str,
    "protocol": str,
    "tier": str,
}

LUSTRE_CONFIG_SPEC = {
    "id": str,
    "capacityGb": int,
    "description": str,
    "filesystem": str,
    "lustre": str,
}


class FlagTypes:
  """Generates flag types for clusters commands.

  Attributes:
    messages: The messages module for the given API version.
    is_alpha: True if the API version is v1alpha.
  """

  def __init__(self, api_version: str) -> None:
    """Initializes FlagTypes.

    Args:
      api_version: The API version string, e.g., 'v1alpha' or 'v1beta'.
    """

    self.messages = api_utils.GetMessagesModule(
        api_utils.GetReleaseTrack(api_version)
    )
    self.is_alpha = api_version == "v1alpha"

  def GetFilestoresObject(self) -> arg_parsers.ArgObject:
    return arg_parsers.ArgObject(
        spec={
            "id": str,
            "name": str,
            "tier": self.messages.NewFilestoreConfig.TierValueValuesEnum,
            "capacityGb": int,
            "fileshare": str,
            "protocol": (
                self.messages.NewFilestoreConfig.ProtocolValueValuesEnum
            ),
            "description": str,
        },
        required_keys=["id", "name", "tier", "capacityGb", "fileshare"],
        enable_shorthand=True,
        repeated=True,
        disable_key_description=True,
    )

  def GetProtoBootDiskType(self) -> arg_parsers.ArgObject:
    spec = {
        "type": str,
        "sizeGb": int,
    }
    if self.is_alpha:
      spec["image"] = str
      spec["storagePools"] = arg_parsers.ArgObject(
          value_type=str, repeated=True
      )
    return arg_parsers.ArgObject(
        spec=spec,
        required_keys=["type", "sizeGb"],
        enable_shorthand=True,
    )

  def GetGcsBucketsObject(self) -> arg_parsers.ArgObject:
    """Returns an ArgObject for parsing GCS bucket configurations."""

    spec = {
        "id": str,
        "name": str,
        "storageClass": (
            self.messages.NewBucketConfig.StorageClassValueValuesEnum
        ),
        "enableAutoclass": bool,
        "enableHNS": bool,
    }
    if self.is_alpha:
      spec["terminalStorageClass"] = (
          self.messages.GcsAutoclassConfig.TerminalStorageClassValueValuesEnum
      )
    return arg_parsers.ArgObject(
        spec=spec,
        required_keys=["id", "name"],
        enable_shorthand=True,
        repeated=True,
        disable_key_description=True,
    )

  def GetOnDemandInstancesObject(self) -> arg_parsers.ArgObject:
    spec = {
        "id": str,
        "zone": str,
        "machineType": str,
    }
    return arg_parsers.ArgObject(
        spec=spec,
        required_keys=["id", "zone", "machineType"],
        enable_shorthand=True,
        repeated=True,
    )

  def GetSpotInstancesObject(self) -> arg_parsers.ArgObject:
    spec = {
        "id": str,
        "zone": str,
        "machineType": str,
        "terminationAction": (
            self.messages.NewSpotInstancesConfig.TerminationActionValueValuesEnum
        ),
    }
    return arg_parsers.ArgObject(
        spec=spec,
        required_keys=["id", "zone", "machineType"],
        enable_shorthand=True,
        repeated=True,
    )

  def GetReservedInstancesObject(self) -> arg_parsers.ArgObject:
    spec = {
        "id": str,
        "reservation": str,
    }
    if self.is_alpha:
      spec.update({
          "reservationBlock": str,
          "reservationSubBlock": str,
      })
    return arg_parsers.ArgObject(
        spec=spec,
        required_keys=["id"],
        enable_shorthand=True,
        repeated=True,
    )

  def GetFlexStartInstancesObject(self) -> arg_parsers.ArgObject:
    spec = {
        "id": str,
        "zone": str,
        "machineType": str,
        "maxDuration": str,
    }
    return arg_parsers.ArgObject(
        spec=spec,
        required_keys=["id", "zone", "machineType", "maxDuration"],
        enable_shorthand=True,
        repeated=True,
    )

  def GetSlurmNodeSetsObject(self) -> arg_parsers.ArgObject:
    """Returns an ArgObject for parsing Slurm Node Sets configurations."""
    if self.is_alpha:
      return arg_parsers.ArgObject(
          spec={
              "id": str,
              "computeId": str,
              "staticNodeCount": int,
              "maxDynamicNodeCount": int,
              "computeInstance": arg_parsers.ArgObject(
                  spec={
                      "startupScript": str,
                      "labels": LABEL,
                      "bootDisk": self.GetProtoBootDiskType(),
                      "startupScriptTimeout": str,
                  },
                  enable_shorthand=True,
              ),
              "containerNodePool": arg_parsers.ArgObject(
                  spec={
                      "resourceLabels": LABEL,
                      "startupScript": str,
                  },
                  enable_shorthand=True,
              ),
              "startupScript": arg_parsers.ArgObject(),
              "labels": LABEL,
              "bootDisk": self.GetProtoBootDiskType(),
              "startupScriptTimeout": str,
              "container-resource-labels": LABEL,
              "container-startup-script": arg_parsers.ArgObject(),
              "type": str,
              "storageConfigs": STORAGE_CONFIG,
          },
          required_keys=["id"],
          enable_shorthand=True,
          repeated=True,
          disable_key_description=True,
      )
    else:
      return arg_parsers.ArgObject(
          spec={
              "id": str,
              "computeId": str,
              "staticNodeCount": int,
              "maxDynamicNodeCount": int,
              "computeInstance": arg_parsers.ArgObject(
                  spec={
                      "startupScript": str,
                      "labels": LABEL,
                      "bootDisk": self.GetProtoBootDiskType(),
                  },
                  enable_shorthand=True,
              ),
              "storageConfigs": STORAGE_CONFIG,
          },
          required_keys=["id"],
          enable_shorthand=True,
          repeated=True,
          disable_key_description=True,
      )

  def _GetSlurmPartitionSpec(self) -> dict[str, Any]:
    spec = {
        "id": str,
        "nodeSetIds": arg_parsers.ArgObject(value_type=str, repeated=True),
    }
    return spec

  def GetSlurmPartitionsObject(self) -> arg_parsers.ArgObject:
    return arg_parsers.ArgObject(
        spec=self._GetSlurmPartitionSpec(),
        required_keys=["id", "nodeSetIds"],
        enable_shorthand=True,
        repeated=True,
    )

  def GetSlurmPartitionsUpdateObject(self) -> arg_parsers.ArgObject:
    return arg_parsers.ArgObject(
        spec=self._GetSlurmPartitionSpec(),
        required_keys=["id"],
        enable_shorthand=True,
        repeated=True,
    )

  def GetSlurmLoginNodeObject(self) -> arg_parsers.ArgObject:
    """Returns an ArgObject for parsing Slurm Login Node configurations."""
    if self.is_alpha:
      return arg_parsers.ArgObject(
          spec={
              "machineType": str,
              "zone": str,
              "count": int,
              "enableOsLogin": bool,
              "enablePublicIps": bool,
              "startupScript": arg_parsers.ArgObject(),
              "labels": LABEL,
              "bootDisk": self.GetProtoBootDiskType(),
              "storageConfigs": STORAGE_CONFIG,
          },
          required_keys=["machineType", "zone"],
          enable_shorthand=True,
          disable_key_description=True,
      )
    else:
      return arg_parsers.ArgObject(
          spec={
              "machineType": str,
              "zone": str,
              "count": int,
              "enableOsLogin": bool,
              "enablePublicIps": bool,
              "startupScript": str,
              "labels": LABEL,
              "bootDisk": self.GetProtoBootDiskType(),
              "storageConfigs": STORAGE_CONFIG,
          },
          required_keys=["machineType", "zone"],
          enable_shorthand=True,
          disable_key_description=True,
      )

  def GetSlurmLoginNodeUpdateObject(self) -> arg_parsers.ArgObject:
    """Returns an ArgObject for parsing Slurm Login Node update configurations."""
    if self.is_alpha:
      return arg_parsers.ArgObject(
          spec={
              "count": int,
              "startupScript": arg_parsers.ArgObject(),
              "bootDisk": self.GetProtoBootDiskType(),
          },
          required_keys=[],
          enable_shorthand=True,
          disable_key_description=True,
      )
    else:
      return arg_parsers.ArgObject(
          spec={
              "count": int,
              "startupScript": str,
              "bootDisk": self.GetProtoBootDiskType(),
          },
          required_keys=[],
          enable_shorthand=True,
          disable_key_description=True,
      )

  def GetClusterFlagType(self) -> dict[str, Any]:
    """Returns cluster flag type for create command."""
    if self.is_alpha:
      new_on_demand_instances_spec = {
          "machineType": str,
          "zone": str,
      }
      new_spot_instances_spec = new_on_demand_instances_spec | {
          "terminationAction": str
      }
      new_flex_start_instances_spec = new_spot_instances_spec | {
          "maxDuration": str
      }
      new_reserved_instances_spec = new_on_demand_instances_spec | {
          "reservation": str,
          "type": str,
          "reservationBlock": str,
          "reservationSubBlock": str,
      }
      bucket_config_spec = {
          "autoclass": arg_parsers.ArgObject(
              spec={
                  "enabled": bool,
                  "terminalStorageClass": str,
              }
          ),
          "bucket": str,
          "hierarchicalNamespace": arg_parsers.ArgObject(
              spec={"enabled": bool}
          ),
          "storageClass": str,
      }
      return {
          "computeResources": arg_parsers.ArgObject(
              key_type=str,
              value_type=arg_parsers.ArgObject(
                  spec={
                      "config": arg_parsers.ArgObject(
                          spec={
                              "newFlexStartInstances": arg_parsers.ArgObject(
                                  spec=new_flex_start_instances_spec
                              ),
                              "newDwsFlexInstances": arg_parsers.ArgObject(
                                  spec=new_flex_start_instances_spec
                              ),
                              "newOnDemandInstances": arg_parsers.ArgObject(
                                  spec=new_on_demand_instances_spec
                              ),
                              "newReservedInstances": arg_parsers.ArgObject(
                                  spec=new_reserved_instances_spec
                              ),
                              "newSpotInstances": arg_parsers.ArgObject(
                                  spec=new_spot_instances_spec
                              ),
                          }
                      ),
                  }
              ),
          ),
          "description": str,
          "labels": LABEL,
          "name": str,
          "networkResources": arg_parsers.ArgObject(
              key_type=str,
              value_type=arg_parsers.ArgObject(
                  spec={
                      "config": arg_parsers.ArgObject(
                          spec={
                              "existingNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "network": str,
                                      "subnetwork": str,
                                  }
                              ),
                              "newNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "description": str,
                                      "network": str,
                                  }
                              ),
                              "newComputeNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "description": str,
                                      "network": str,
                                  }
                              ),
                              "existingComputeNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "network": str,
                                      "subnetwork": str,
                                  }
                              ),
                          }
                      ),
                  }
              ),
          ),
          "orchestrator": arg_parsers.ArgObject(
              spec={
                  "slurm": arg_parsers.ArgObject(
                      spec={
                          "defaultPartition": str,
                          "loginNodes": arg_parsers.ArgObject(
                              spec={
                                  "count": int,
                                  "enableOsLogin": bool,
                                  "enablePublicIps": bool,
                                  "labels": LABEL,
                                  "machineType": str,
                                  "startupScript": arg_parsers.ArgObject(),
                                  "storageConfigs": STORAGE_CONFIG,
                                  "zone": str,
                                  "bootDisk": self.GetProtoBootDiskType(),
                              }
                          ),
                          "nodeSets": arg_parsers.ArgObject(
                              spec={
                                  "id": str,
                                  "maxDynamicNodeCount": int,
                                  "staticNodeCount": int,
                                  "storageConfigs": STORAGE_CONFIG,
                                  "computeId": str,
                                  "computeInstance": arg_parsers.ArgObject(
                                      spec={
                                          "startupScript": (
                                              arg_parsers.ArgObject()
                                          ),
                                          "labels": LABEL,
                                          "bootDisk": (
                                              self.GetProtoBootDiskType()
                                          ),
                                          "startupScriptTimeout": str,
                                      }
                                  ),
                                  "containerNodePool": arg_parsers.ArgObject(
                                      spec={}
                                  ),
                              },
                              repeated=True,
                          ),
                          "partitions": arg_parsers.ArgObject(
                              spec={
                                  "id": str,
                                  "nodeSetIds": arg_parsers.ArgObject(
                                      value_type=str,
                                      repeated=True,
                                  ),
                              },
                              repeated=True,
                          ),
                          "prologBashScripts": arg_parsers.ArgList(),
                          "epilogBashScripts": arg_parsers.ArgList(),
                      }
                  ),
              }
          ),
          "storageResources": arg_parsers.ArgObject(
              key_type=str,
              value_type=arg_parsers.ArgObject(
                  spec={
                      "config": arg_parsers.ArgObject(
                          spec={
                              "existingBucket": arg_parsers.ArgObject(
                                  spec={"bucket": str}
                              ),
                              "existingFilestore": arg_parsers.ArgObject(
                                  spec={"filestore": str}
                              ),
                              "existingLustre": arg_parsers.ArgObject(
                                  spec={"lustre": str}
                              ),
                              "newBucket": arg_parsers.ArgObject(
                                  spec=bucket_config_spec
                              ),
                              "newFilestore": arg_parsers.ArgObject(
                                  spec=FILESTORE_CONFIG_SPEC
                              ),
                              "newLustre": arg_parsers.ArgObject(
                                  spec=LUSTRE_CONFIG_SPEC
                                  | {"perUnitStorageThroughput": int}
                              ),
                          },
                      )
                  }
              ),
          ),
      }
    else:  # Beta
      new_on_demand_instances_beta_spec = {
          "machineType": str,
          "zone": str,
      }
      new_spot_instances_beta_spec = new_on_demand_instances_beta_spec | {
          "terminationAction": str
      }
      new_flex_start_instances_beta_spec = new_on_demand_instances_beta_spec | {
          "maxDuration": str
      }
      new_reserved_instances_beta_spec = {
          "reservation": str,
      }
      bucket_config_beta_spec = {
          "autoclass": arg_parsers.ArgObject(
              spec={
                  "enabled": bool,
              }
          ),
          "bucket": str,
          "hierarchicalNamespace": arg_parsers.ArgObject(
              spec={"enabled": bool}
          ),
          "storageClass": str,
      }
      return {
          "computeResources": arg_parsers.ArgObject(
              key_type=str,
              value_type=arg_parsers.ArgObject(
                  spec={
                      "config": arg_parsers.ArgObject(
                          spec={
                              "newFlexStartInstances": arg_parsers.ArgObject(
                                  spec=new_flex_start_instances_beta_spec
                              ),
                              "newDwsFlexInstances": arg_parsers.ArgObject(
                                  spec=new_flex_start_instances_beta_spec
                              ),
                              "newOnDemandInstances": arg_parsers.ArgObject(
                                  spec=new_on_demand_instances_beta_spec
                              ),
                              "newReservedInstances": arg_parsers.ArgObject(
                                  spec=new_reserved_instances_beta_spec
                              ),
                              "newSpotInstances": arg_parsers.ArgObject(
                                  spec=new_spot_instances_beta_spec
                              ),
                          }
                      ),
                  }
              ),
          ),
          "description": str,
          "labels": LABEL,
          "name": str,
          "networkResources": arg_parsers.ArgObject(
              key_type=str,
              value_type=arg_parsers.ArgObject(
                  spec={
                      "config": arg_parsers.ArgObject(
                          spec={
                              "existingNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "network": str,
                                      "subnetwork": str,
                                  }
                              ),
                              "newNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "description": str,
                                      "network": str,
                                  }
                              ),
                              "newComputeNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "description": str,
                                      "network": str,
                                  }
                              ),
                              "existingComputeNetwork": arg_parsers.ArgObject(
                                  spec={
                                      "network": str,
                                      "subnetwork": str,
                                  }
                              ),
                          }
                      ),
                  }
              ),
          ),
          "orchestrator": arg_parsers.ArgObject(
              spec={
                  "slurm": arg_parsers.ArgObject(
                      spec={
                          "defaultPartition": str,
                          "loginNodes": self.GetSlurmLoginNodeObject(),
                          "nodeSets": self.GetSlurmNodeSetsObject(),
                          "partitions": self.GetSlurmPartitionsObject(),
                          "prologBashScripts": arg_parsers.ArgList(),
                          "epilogBashScripts": arg_parsers.ArgList(),
                      }
                  ),
              }
          ),
          "storageResources": arg_parsers.ArgObject(
              key_type=str,
              value_type=arg_parsers.ArgObject(
                  spec={
                      "config": arg_parsers.ArgObject(
                          spec={
                              "existingBucket": arg_parsers.ArgObject(
                                  spec={"bucket": str}
                              ),
                              "existingFilestore": arg_parsers.ArgObject(
                                  spec={"filestore": str}
                              ),
                              "existingLustre": arg_parsers.ArgObject(
                                  spec={"lustre": str}
                              ),
                              "newBucket": arg_parsers.ArgObject(
                                  spec=bucket_config_beta_spec
                              ),
                              "newFilestore": arg_parsers.ArgObject(
                                  spec=FILESTORE_CONFIG_SPEC
                              ),
                              "newLustre": arg_parsers.ArgObject(
                                  spec=LUSTRE_CONFIG_SPEC
                                  | {"perUnitStorageThroughput": int}
                              ),
                          },
                      )
                  }
              ),
          ),
      }



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Database Migration Service connection profiles API."""


from typing import Optional

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.database_migration import api_util


class ObjectsClient(object):
  """Client for migration job objects service in the API."""

  def __init__(self, release_track, location: Optional[str] = None):
    self._api_version = api_util.GetApiVersion(release_track)
    self.client = api_util.GetClientInstance(release_track, location=location)
    self.messages = api_util.GetMessagesModule(release_track)
    self._service = self.client.projects_locations_migrationJobs_objects
    self.resource_parser = api_util.GetResourceParser(release_track)
    self._release_track = release_track

  def List(self, migration_job_ref, args):
    """Get the list of objects in a migration job.

    Args:
      migration_job_ref: The migration job for which to list objects.
      args: parsed command line arguments

    Returns:
      An iterator over all the matching migration job objects.
    """
    list_req_type = (
        self.messages.DatamigrationProjectsLocationsMigrationJobsObjectsListRequest
    )
    list_req = list_req_type(parent=migration_job_ref.RelativeName())

    return list_pager.YieldFromList(
        service=self._service,
        request=list_req,
        limit=args.limit,
        batch_size=args.page_size,
        field='migrationJobObjects',
        batch_size_attribute='pageSize',
    )

  def Lookup(self, migration_job_ref, args):
    """Lookup a migration job object.

    Args:
      migration_job_ref: The migration job name to which the object belongs.
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      MigrationJobObject: the looked up  migration job object.
    """
    source_object_identifier = self.messages.SourceObjectIdentifier(
        database=args.database,
        schema=args.schema,
        table=args.table,
        type=self.GetType(args)
    )

    lookup_req_type = (
        self.messages.DatamigrationProjectsLocationsMigrationJobsObjectsLookupRequest
    )
    lookup_req = lookup_req_type(
        lookupMigrationJobObjectRequest=self.messages.LookupMigrationJobObjectRequest(
            sourceObjectIdentifier=source_object_identifier
        ),
        parent=migration_job_ref.RelativeName(),
    )
    return self._service.Lookup(lookup_req)

  def GetType(self, args):
    if  args.IsKnownAndSpecified('type'):
      return self.messages.SourceObjectIdentifier.TypeValueValuesEnum.lookup_by_name(
          args.type
          )
    return (self.messages.SourceObjectIdentifier.TypeValueValuesEnum
            .lookup_by_name('DATABASE'))



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags module for Data Lineage commands."""

import argparse

ArgumentParser = argparse.ArgumentParser


def AddParentArgs(parser: ArgumentParser) -> None:
  """Adds mutually exclusive args for the parent resource to the parser."""
  parent_group = parser.add_mutually_exclusive_group(required=False)
  parent_group.add_argument(
      '--project',
      help=(
          'Project ID or number. If none of --project, --folder, or '
          '--organization are provided, the current project will be used.'
      ),
  )
  parent_group.add_argument('--folder', help='Folder ID.')
  parent_group.add_argument('--organization', help='Organization ID.')



# Copyright 2005 Google Inc.
# All Rights Reserved.
#
# Author: roth@google.com (Mark D. Roth), dgreiman@google.com (Douglas Greiman)

"""Access to google3 build data.

This module allows access to build data produced by Blaze when it compiles a
.par file. This is the Python analogue of the C++ build stamp headers and the
Java deploy jar's build-data.properties file. Data may include information
such as the CL and time at which the target was last built, the Sponge ID of the
build which created the target, and the user, client, and workspace from which
the build was invoked.
"""

import io
import sys
import time
from typing import Optional

from pyglib import resources

try:
  from google.base.python.clif import googleinit  # pylint: disable=g-import-not-at-top  # pytype: disable=import-error
except ImportError:
  googleinit = None


# Global data
_build_dict = None


def _ParseBuildData(filename: str) -> dict[str, str]:
  """Read Blaze-stamped build data from external file.

  Uses default values if build data file not available.

  Args:
    filename:  The path to the resource from which to read.

  Returns:
    A dictionary of build data info.
  """

  # Start with empty defaults
  build_data = {
      'BUILDINFO': '',
      'BUILDLABEL': '',
      'BUILDTOOL': '',
      'CHANGELIST': '-1',
      'BASELINECHANGELIST': '-1',
      'CLIENTNAME': '',
      'CLIENTSTATUS': '-2',
      'DEPOTPATH': '',
      'PAROPTIONS': '',
      'PLATFORM': '',
      'TARGET': '',
      'TIMESTAMP': '',
      'VERSIONMAP': '',
  }

  # Read generated build info if available
  try:
    build_data_str = resources.GetResource(filename).decode()
    # Parse as key/value pairs
    for line in build_data_str.splitlines():
      # Skip comments, blanks, and syntax errors
      if line.startswith('#') or not line.strip() or ':' not in line:
        continue

      key, value = line.split(':', 1)
      build_data[key] = value
  except IOError:
    # Build data not available. Most of time we don't need this info. However,
    # if you do need build data, you might want to check if --nostamp is
    # provided in your blazerc file.
    pass
  except Exception as unexpected_error:  # pylint: disable=broad-except
    # Tolerate other errors as well, but dump the error info to the log.
    #
    # NOTE: Errors parsing build data are somewhat more common when using
    # g3process with a .par file on borg since forked processes will share the
    # file descriptor (and file offset) for the .par file which can mess up file
    # reads.  The resulting ValueError is safe to ignore since the BUILD info
    # for the child processes is the same as for the parent and will have been
    # logged at startup.  See b/117880794 for more info.
    print('\nUnexpected error parsing build data from \'{}\': {}\n'.format(
        filename, unexpected_error), file=sys.stderr)

  return build_data


def _GetBuildData() -> dict[str, str]:
  """Read build data from external file, if not already read."""
  global _build_dict  # pylint: disable=global-statement
  if _build_dict is None:
    _build_dict = _ParseBuildData('google3/pyglib/build_data.txt')
  return _build_dict


def BuildInfo() -> str:
  """Return user, host, and directory of builder, as string."""
  return _GetBuildData().get('BUILDINFO', '')


def BuildLabel() -> str:
  """Return build label (passed to make-{opt,dbg} -l) as string."""
  return _GetBuildData().get('BUILDLABEL', '')


def BuildTool() -> str:
  """Return the build tool as string if we know it."""
  return _GetBuildData().get('BUILDTOOL', '')


def BuildID() -> str:
  """Return the build ID of the invocation which built the binary, if known."""
  return _GetBuildData().get('BUILDID', '')


def Changelist() -> int:
  """Return client workspace changelist, as int."""
  cl = _GetBuildData().get('CHANGELIST', '')
  try:
    changelist = int(cl.strip())
  except ValueError:
    changelist = -1
  return changelist


def BaselineChangelist() -> int:
  """Return the mainline CL number from which this was built.

  Returns:
    If built from a branch, the CL of mainline (//depot/google3) on which the
    branch was based, otherwise the same as `Changelist()`.
  """
  cl = _GetBuildData().get('BASELINECHANGELIST', '')
  try:
    changelist = int(cl.strip())
  except ValueError:
    changelist = -1
  return changelist


def CitcSnapshot() -> str:
  """Return CitC Snapshot number, as string."""
  citc_snapshot = _GetBuildData().get('CITCSNAPSHOT', '')
  if citc_snapshot == 'null':
    return ''
  return citc_snapshot


def CitcWorkspaceId() -> str:
  """Return CitC Workspace ID, as string, like whoami/123."""
  citc_workspace_id = _GetBuildData().get('CITCWORKSPACEID', '')
  return '' if citc_workspace_id == 'null' else citc_workspace_id


def SourceUri() -> str:
  """Return go/source-uri, as string."""
  return _GetBuildData().get('SOURCEURI', '')


def ClientName() -> str:
  """Return Perforce client name, as string."""
  return _GetBuildData().get('CLIENTNAME', '')


def ClientStatus() -> int:
  """Return Perforce client status, as int."""
  tmp_str = _GetBuildData().get('CLIENTSTATUS', '')
  try:
    status = int(tmp_str)
  except ValueError:
    status = -1
  return status


def DepotPath() -> str:
  """Return Perforce depot path, as string."""
  return _GetBuildData().get('DEPOTPATH', '')


def ParOptions() -> str:
  """Return list of autopar options, as string."""
  return _GetBuildData().get('PAROPTIONS', '')


def Platform() -> str:
  """Return google platform as string."""
  return _GetBuildData().get('PLATFORM', '')


def Target() -> str:
  """Return build target as string."""
  return _GetBuildData().get('TARGET', '')


def Timestamp() -> int:
  """Return timestamp in seconds since the Epoch, as int."""
  ts = _GetBuildData().get('TIMESTAMP', '')
  try:
    timestamp = int(ts.strip())
  except ValueError:
    timestamp = -1
  return timestamp


def TimestampAscii() -> str:
  """Return timestamp as a string: May 12 2009 13:59:22 (1242161962)."""
  timestamp = Timestamp()
  timestamp_str = ('%s (%d)' %
                   (time.asctime(time.localtime(timestamp)), timestamp))
  return timestamp_str


def Verifiable() -> int:
  """Return the value of VERIFIABLE, as integer."""
  verifiable = _GetBuildData().get('VERIFIABLE', '0')
  try:
    return int(verifiable)
  except ValueError:
    return 0


def VersionMap() -> str:
  """Return the map from VERSION_MAP as string."""
  vm = _GetBuildData().get('VERSIONMAP', '')
  if 'client_view {' in vm:
    return vm[:vm.index('client_view {') - 1].strip(',')
  else:
    return vm


def FullVersionMap() -> str:
  """Return VERSION_MAP as string, including the client_view."""
  return _GetBuildData().get('VERSIONMAP', '')


def BuildDebugMode() -> Optional[bool]:
  """Returns whether NDEBUG was defined when it has C++ deps, None otherwise."""
  return not googleinit.IsNDebugDefined() if googleinit else None


def ClientInfo() -> str:
  """Return Perforce client changelist and status as descriptive string."""
  info = ''
  cl = Changelist()
  vm = VersionMap()
  citc_snapshot = CitcSnapshot()
  build_client = ClientName()
  if vm:
    cs = ClientStatus()
    dp = DepotPath()
    if cs == 1:
      status_info = ' in a mint Components client based on %s' % dp
    elif cs == 0:
      status_info = ' in a modified Components client based on %s' % dp
    else:
      status_info = ' in a possibly-modified Components client'
    if citc_snapshot:
      status_info += ' in CitC workspace %s at snapshot %s' % (
          build_client, citc_snapshot)
    info = 'version map "%s"%s' % (vm, status_info)
    if cl > 0 and '/branches/' in dp:
      info += ' at branch changelist %d' % cl
  else:
    if cl == -1:
      info = ''
    elif cl == 0:
      info = 'unknown changelist'
    else:
      cs = ClientStatus()
      dp = DepotPath()
      if cs == 1:
        status_info = ' in a mint client based on %s' % dp
      elif cs == 0:
        status_info = ' in a modified client based on %s' % dp
      else:
        status_info = ' possibly in a modified client'
      if citc_snapshot:
        status_info += ' in CitC workspace %s at snapshot %s' % (
            build_client, citc_snapshot)
      info = 'changelist %d%s' % (cl, status_info)

  return info


def BuildData() -> str:
  """Return all build data as a nicely formatted string."""
  buf = io.StringIO()

  buf.write('Built on %s\n' % TimestampAscii())

  buf.write('Built by %s\n' % BuildInfo())

  buf.write('Built as %s\n' % Target())

  buildid = BuildID()
  if buildid:
    buf.write('Build ID: %s\n' % buildid)

  clientinfo = ClientInfo()
  if clientinfo:
    buf.write('Built from %s\n' % clientinfo)

  buildlabel = BuildLabel()
  if buildlabel:
    buf.write('Build label: %s\n' % buildlabel)

  buf.write('Build platform: %s\n' % Platform())

  buildtool = BuildTool()
  if buildtool:
    buf.write('Build tool: %s\n' % buildtool)

  paropts = ParOptions()
  if paropts:
    buf.write('Built with par options %s\n' % paropts)

  gil_enabled = getattr(sys, '_is_gil_enabled', lambda: False)()
  if gil_enabled:
    gil_enabled_str = ''
  else:
    gil_enabled_str = ' (Python free-threading - no GIL)'
  python_executable = sys.executable if sys.executable else 'embedded.'
  buf.write(
      f'Currently running under Python {sys.version}:'
      f' {python_executable}{gil_enabled_str}.\n'
  )

  data_str = buf.getvalue()
  buf.close()

  return data_str
